package cn.com.qmth.dp.examcloud.oe.modules.debug;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.bson.types.ObjectId;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.nodes.Node;
import org.jsoup.nodes.TextNode;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;

import cn.com.qmth.dp.examcloud.oe.entity.question.PaperDetailUnit;
import cn.com.qmth.dp.examcloud.oe.entity.question.QuesOption;
import cn.com.qmth.dp.examcloud.oe.entity.question.Question;

@Service
public class DebugService {
    


    public static final String QUESOPS = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";

    public static final String ELEMENT_TYPE_RID = "imageId";

    public static final String ELEMENT_TYPE_TEXT = "text";

    public static final String ELEMENT_TYPE_IMG = "image";
    
    public static final String ELEMENT_TYPE_DOCTAG = "doctag";


	@Autowired
	MongoTemplate mongoTemplate;

	public void start() {
		Query query = new Query();
		query.addCriteria(Criteria.where("paper.$id").is(new ObjectId("64ddd05a391d2033f36b0a20")));
		List<PaperDetailUnit> ps = mongoTemplate.find(query, PaperDetailUnit.class, "paperDetailUnit");
		ExportPaperUtil.disposePaperDoc(ps);
//		for(PaperDetailUnit p:ps) {
//			disposeQuestion(p.getQuestion());
//		}
	}
	
	private static void disposeQuestion(Question qes) {
        if (qes != null) {

            //处理选项
            if (qes.getQuesOptions() != null && qes.getQuesOptions().size() > 0) {
                int index = 0;
                for (QuesOption qo : qes.getQuesOptions()) {
                    List<JSection> slist2 = getSections(qo.getOptionBody());
                    if (slist2 != null && slist2.size() > 0) {
                        SectionElement se = new SectionElement();
                        se.setType(ELEMENT_TYPE_TEXT);
                        se.setValue(QUESOPS.charAt(index) + ".");
                        index++;
                        slist2.get(0).getElements().add(0, se);
//                        qo.setOptionBodyWord(getQuestionDoc(slist2));
                    }else {
                    	System.out.println("******");
                    }

                }
            }
            
            if (qes.getSubQuestions() != null && qes.getSubQuestions().size() > 0) {
                for (Question sunqes : qes.getSubQuestions()) {
                    disposeQuestion(sunqes);// 递归处理套题
                }
            }
        }
    }
	private static List<JSection> getSections(String html) {
        return getSections(html, false);
    }

    private static List<JSection> getSections(String html, boolean diposeFillBlank) {
        if (StringUtils.isBlank(html)) {
            return getEmptySections();
        }
        html = html.trim();
        if (!html.startsWith("<p>")) {
            html = "<p>" + html + "</p>";
        }
        List<JSection> ss = new ArrayList<JSection>();
        Document doc = Jsoup.parse(html);
        Element b = doc.body();
        for (Node e : b.childNodes()) {
            if ("p".equals(e.nodeName()) && e.childNodeSize() > 0) {
                JSection s = new JSection();
                List<SectionElement> ses = new ArrayList<SectionElement>();
                s.setElements(ses);
                for (Node ce : e.childNodes()) {
                    getSectionElement(ce, ses, diposeFillBlank);
                }
                ss.add(s);
            }
        }
        return ss;

    }
    
    private static List<JSection> getEmptySections(){
    	List<JSection> ss = new ArrayList<JSection>();
    	JSection s = new JSection();
        List<SectionElement> ses = new ArrayList<SectionElement>();
        s.setElements(ses);
        SectionElement se=new SectionElement();
        se.setType(ELEMENT_TYPE_TEXT);
        se.setValue("");
        ses.add(se);
        ss.add(s);
        return ss;
    }

    private static void getSectionElement(Node ce, List<SectionElement> ses, boolean diposeFillBlank) {
        if ("span".equals(ce.nodeName()) && ce.childNodeSize() > 0) {
            for (Node e : ce.childNodes()) {
                getSectionElement(e, ses, diposeFillBlank);
            }
        } else {
            if (ce instanceof TextNode) {
            	SectionElement se = new SectionElement();
                TextNode tn = (TextNode) ce;
                se.setType(ELEMENT_TYPE_TEXT);
                String text = tn.text();
                if (diposeFillBlank) {
                    text = text.replaceAll("###", "______").replaceAll("##", "______");
                }
                text = text.replaceAll("&", "&amp;").replaceAll("<", "&lt;").replaceAll(">", "&gt;").replaceAll("\u0000", " ");
                se.setValue(text);
                ses.add(se);
            } else if (ce instanceof Element) {
            	SectionElement se = new SectionElement();
                if ("img".equals(ce.nodeName())) {
                    se.setType(ELEMENT_TYPE_IMG);
                    se.setValue(ce.attr("src"));
//                    setImageSize(se, ce);
                } else if ("br".equals(ce.nodeName())) {
                    se.setType(ELEMENT_TYPE_DOCTAG);
                    se.setValue("<w:r w:rsidRPr=\"003A4B1C\"><w:br/></w:r>");
                } else {
                    Element el = (Element) ce;
                    se.setType(ELEMENT_TYPE_TEXT);
                    se.setValue(el.text());
                }
                ses.add(se);
            }
        }
    }

}
